/*
 * indentDifference.c
 * Created by M.Miyazaki [2013.01.04]
 */

#include "indentDifference.h"

/**
 * For Indent Informations.
 */
IndentListNode *indentRootNode = NULL;

/**
 * Number of spaces that indicates tab length.
 */
int tabSpaces = DEFAULT_TAB_SPACES;

/**
 * Set tab spaces.
 */
void setTabSpaces(int numberOfSpaces)
{
	tabSpaces = numberOfSpaces;
	return;
}

/**
 * Register an information of indent.
 */
int registerIndent(int indentLevel, int indentSpaces, int indentTabs)
{
	IndentListNode *aNode;
	
	aNode = (IndentListNode *)malloc(sizeof(IndentListNode));
	if (aNode == NULL)
	{
		return -1;
	}
	
	aNode->next         = indentRootNode;
	aNode->indentLevel  = indentLevel;
	aNode->indentSpaces = indentSpaces;
	aNode->indentTabs   = indentTabs;
	
	indentRootNode = aNode;
	
	return 0;
}

/**
 * Calculate number of indent space.
 */
double indentFrom(IndentListNode *aNode)
{
	return (double)(aNode->indentTabs * tabSpaces + aNode->indentSpaces);
}

/**
 * Calculate number of indent space. It is normalized.
 */
double normalIndentFrom(IndentListNode *aNode)
{
	return indentFrom(aNode) / (double)aNode->indentLevel;
}

/**
 * Calculate 'standardIndent'
 */
double calculateStandardIndent(void)
{
	double returnValue = -1.0, normalIndent;
	int isError = FALSE, isNewValue, maxCount;
	HistogramNode *histogramRootNode = NULL, *aNode;
	IndentListNode *indentNode;
	
	if (indentRootNode == NULL) isError = TRUE;
	
	indentNode = indentRootNode;
	while (indentNode != NULL)
	{
		if (indentNode->indentLevel == 0)
		{
			// Ingore indentNode.
			indentNode = indentNode->next;
			continue;
		}
		
		normalIndent = (double)normalIndentFrom(indentNode);
		isNewValue = TRUE;
		aNode = histogramRootNode;
		while (aNode != NULL)
		{
			// Find equivalent value in list.
			if (pow(aNode->normalIndent - normalIndent, 2) < INDENT_ACCURACY)
			{
				aNode->count += 1;
				isNewValue = FALSE;
				break;
			}
			
			aNode = aNode->next;
		}
		
		if (isNewValue == TRUE)
		{
			// Register new value.
			aNode = (HistogramNode *)malloc(sizeof(HistogramNode));
			if (aNode == NULL)
			{
				isError = TRUE;
				break;
			}
			
			aNode->next         = histogramRootNode;
			aNode->normalIndent = normalIndent;
			aNode->count        = 1;
			
			histogramRootNode = aNode;
		}
		
		indentNode = indentNode->next;
	}
	
	if (isError == FALSE)
	{
		// Find the standard indent.
		maxCount = 0;
		aNode = histogramRootNode;
		while (aNode != NULL)
		{
			if (maxCount < aNode->count)
			{
				maxCount    = aNode->count;
				returnValue = aNode->normalIndent;
			}
			
			aNode = aNode->next;
		}
		
	}
	
	// Deallocate.
	while (histogramRootNode != NULL)
	{
		aNode = histogramRootNode->next;
		free(histogramRootNode);
		histogramRootNode = aNode;
	}
	
	return returnValue;
}

/**
 * Calculate 'indentDifference'.
 */
double calculateIndentDifference(void)
{
	double difference, standardIndent, sum = 0.0;
	int count = 0;
	IndentListNode *aNode;
	
	standardIndent = calculateStandardIndent();
	
	aNode = indentRootNode;
	while (aNode != NULL)
	{
		sum += fabs((standardIndent * aNode->indentLevel) - indentFrom(aNode));
		count++;
		//printf(">>> INDENT : %5d %5d %5d\n", aNode->indentLevel, aNode->indentSpaces, aNode->indentTabs);
		aNode = aNode->next;
	}
	
	difference = sum / count;
	
	return difference;
}

/**
 * Deallocate all nodes of indent list.
 */
void freeIndentList(void)
{
	IndentListNode *aNode;
	
	aNode = indentRootNode;
	while (aNode != NULL)
	{
		indentRootNode = aNode->next;
		free(aNode);
		aNode = indentRootNode;
	}
	
	return;
}
