/*
 * main.c
 * Created by M.Miyazaki [2012.12.28]
 */

#include "defs.h"

/**
 * Number of spaces that indicates tab length.
 */
#define TAB_SPACES 4


/**
 * For Debug
 */
int debugMode = FALSE;

/**
 * For Effective LOC. This excludes Blanks, Brackets, and Comments.
 */
int effectiveLOC;
int isEffectiveLine;

/**
 * For Physical LOC. This includes Blanks, Brackets, and Comments.
 */
int physicalLOC;

/**
 * For Logical LOC. This indicates a number of statements.
 */
int logicalLOC;

/**
 * For Comment Lines.
 */
int commentLines;

/**
 * For Number of C Functions.
 */
int numberOfFunctions;

/**
 * For Average of Identifier's length.
 */
double averageLengthOfId;
int numberOfId;
int sumOfIdLength;

/**
 * For Difference of indentation.
 */
double indentDifference;
int indentSpaces;
int indentTabs;
int indentLevel;
int isInIndent;

/**
 * For Standard Indent.
 */
double standardIndent;

/**
 * For Percentage of Comment.
 */
double commentRatio;

/**
 * Example program.
 */
int main(void)
{
	
	initializeMetricsVariables();
	setTabSpaces(TAB_SPACES);
	
	if (yyparse() != 0) {
		actionOnParseError();
	}
	else
	{
		actionOnParseSuccess();
	}
	
	printMetricsResult();
	
	freeTypedefDictionary();
	freeIndentList();
	
	return(EXIT_SUCCESS);
}

/**
 * Initialize variables
 */
void initializeMetricsVariables(void)
{
	effectiveLOC      = 0;
	isEffectiveLine   = FALSE;
	physicalLOC       = 0;
	logicalLOC        = 0;
	commentLines      = 0;
	numberOfFunctions = 0;
	averageLengthOfId = 0.0;
	indentDifference  = 0.0;
	indentSpaces      = 0;
	indentTabs        = 0;
	indentLevel       = 0;
	isInIndent        = TRUE;
	standardIndent    = 0.0;
	commentRatio      = 0.0;
	
	return;
}

/**
 * Action on parse error.
 */
void actionOnParseError(void)
{
	effectiveLOC      = -1;
	physicalLOC       = -1;
	logicalLOC        = -1;
	commentLines      = -1;
	numberOfFunctions = -1;
	averageLengthOfId = -1.0;
	indentDifference  = -1.0;
	standardIndent    = -1.0;
	commentRatio      = -1.0;
	
	return;
}

/**
 * Action on parse success.
 */
void actionOnParseSuccess()
{
	physicalLOC++;   // Include last line.
	averageLengthOfId = (double)sumOfIdLength / (double)numberOfId;
	indentDifference  = calculateIndentDifference();
	standardIndent    = calculateStandardIndent();
	commentRatio      = (double)commentLines / (double)physicalLOC * 100.0;
	
	return;
}

/**
 * Print out metrics results.
 */
void printMetricsResult(void)
{
	char formattedString[1024], *base64String;
	
	printf("{");
	
	// Lines Of Code (Effective LOC)
	sprintf(formattedString, "%d", effectiveLOC);
	base64String = encodeBase64(formattedString);
	printf("\"linesOfCode\" : \"%s\", ", base64String);
	free(base64String);
	
	// Effective LOC
	sprintf(formattedString, "%d", effectiveLOC);
	base64String = encodeBase64(formattedString);
	printf("\"effectiveLOC\" : \"%s\", ", base64String);
	free(base64String);
	
	// Physical LOC
	sprintf(formattedString, "%d", physicalLOC);
	base64String = encodeBase64(formattedString);
	printf("\"physicalLOC\" : \"%s\", ", base64String);
	free(base64String);
	
	// Logical LOC
	sprintf(formattedString, "%d", logicalLOC);
	base64String = encodeBase64(formattedString);
	printf("\"logicalLOC\" : \"%s\", ", base64String);
	free(base64String);
	
	// Comment Lines
	sprintf(formattedString, "%d", commentLines);
	base64String = encodeBase64(formattedString);
	printf("\"commentLines\" : \"%s\", ", base64String);
	free(base64String);
	
	// Number Of Functions
	sprintf(formattedString, "%d", numberOfFunctions);
	base64String = encodeBase64(formattedString);
	printf("\"numberOfFunctions\" : \"%s\", ", base64String);
	free(base64String);
	
	// Average Length Of Id
	sprintf(formattedString, "%.10lf", averageLengthOfId);
	base64String = encodeBase64(formattedString);
	printf("\"averageLengthOfId\" : \"%s\", ", base64String);
	free(base64String);
	
	// Indent Difference
	sprintf(formattedString, "%.10lf", indentDifference);
	base64String = encodeBase64(formattedString);
	printf("\"indentDifference\" : \"%s\", ", base64String);
	free(base64String);
	
	// Standard Indent
	sprintf(formattedString, "%.10lf", standardIndent);
	base64String = encodeBase64(formattedString);
	printf("\"standardIndent\" : \"%s\", ", base64String);
	free(base64String);
	
	// Comment Ratio
	sprintf(formattedString, "%.10lf", commentRatio);
	base64String = encodeBase64(formattedString);
	printf("\"commentRatio\" : \"%s\"", base64String);
	free(base64String);
	
	printf("}");
	
	return;
}
