/*
 * typedefDictionary.c
 * Created by M.Miyazaki [2012.12.25]
 * Modified by M.Miyazaki [2012.12.28]
 */

#include "typedefDictionary.h"


/**
 * Initialize of dictionary's root element.
 */
DictionaryElement *firstElement = NULL;

/**
 * Initialize of typedefState.
 */
enum TypedefState typedefState = Initial;

/**
 * An integer that indicates level of current scope.
 */
int currentScopeLevel = 0;

/**
 * Check the input 'aString' is typedef-name or not.
 * 0 is false. 1 is true.
 */
int isTypedefName(const char *aString)
{
	int returnValue = 0;
	DictionaryElement *currentElement;
	
	currentElement = firstElement;
	while (currentElement != NULL)
	{
		int compareResult = strcmp(currentElement->typedefName, aString);
		if (compareResult == 0)
		{
			returnValue = 1;
			break;
		}
		currentElement = currentElement->next;
	}
	
	return returnValue;
}

/**
 * Register a typedef-name.
 */
int registerTypedefName(const char *typedefName, int scopeLevel)
{
	char *typedefNameForElement;
	DictionaryElement *anElement;
	DictionaryElement *currentElement, *previousElement;
	
	typedefNameForElement = (char *)malloc(sizeof(char *) * (strlen(typedefName) + 1));
	if (typedefNameForElement == NULL)
	{
		return -1;
	}
	strcpy(typedefNameForElement, typedefName);
	
	anElement = (DictionaryElement *)malloc(sizeof(DictionaryElement));
	if (anElement == NULL)
	{
		free(typedefNameForElement);
		return -1;
	}
	
	previousElement = NULL;
	currentElement  = firstElement;
	while (currentElement != NULL && currentElement->scopeLevel > scopeLevel)
	{
		previousElement = currentElement;
		currentElement  = currentElement->next;
	}
	
	anElement->next        = currentElement;
	anElement->typedefName = typedefNameForElement;
	anElement->scopeLevel  = scopeLevel;
	
	if (previousElement == NULL)
	{
		firstElement = anElement;
	}
	else
	{
		previousElement->next = anElement;
	}
	
	return 0;
}

/**
 * Delete TypedefName elements.
 */
int deleteTypedefNameAt(int scopeLevel)
{
	int countOfDeletedElements = 0;
	DictionaryElement *nextElement;
	
	while (firstElement != NULL)
	{
		if (firstElement->scopeLevel >= scopeLevel)
		{
			nextElement = firstElement->next;
			free(firstElement->typedefName);
			free(firstElement);
			countOfDeletedElements++;
			firstElement = nextElement;
		}
		else
		{
			break;
		}
	}
	
	return countOfDeletedElements;
}

/**
 * Deallocate typedef-name dictionary.
 */
int freeTypedefDictionary(void)
{
	int countOfDeletedElements = 0;
	DictionaryElement *nextElement;
	
	if (firstElement == NULL)
	{
		return 0;
	}
	
	while (firstElement != NULL)
	{
		nextElement = firstElement->next;
		free(firstElement->typedefName);
		free(firstElement);
		countOfDeletedElements++;
		firstElement = nextElement;
	}
	
	return countOfDeletedElements;
}

/**
 * Example program.
 */
/*
int main(int argumentCount, char* argumentValues[])
{
	if (registerTypedefName("hoge", 1) == -1)
	{
		printf("Cannot registered 'hoge' at scope 1.");
		return -1;
	}
	printf("Registered 'hoge' at scope 1.\n");
	
	if (registerTypedefName("foo", 1) == -1)
	{
		printf("Cannot registered 'foo' at scope 1.");
		return -1;
	}
	printf("Registered 'foo' at scope 1.\n");
	
	if (registerTypedefName("hoge", 2) == -1)
	{
		printf("Cannot registered 'hoge' at scope 2.");
		return -1;
	}
	printf("Registered 'hoge' at scope 2.\n");
	
	printf("hoge is %s.\n", isTypedefName("hoge") ? "registered" : "not registered");
	printf("bar is %s.\n", isTypedefName("bar") ? "registered" : "not registered");
	printf("foo is %s.\n", isTypedefName("foo") ? "registered" : "not registered");
	
	deleteTypedefNameAt(2);
	printf("Deleted 'typedef-name's at scope 2.\n");
	
	printf("hoge is %s.\n", isTypedefName("hoge") ? "registered" : "not registered");
	printf("bar is %s.\n", isTypedefName("bar") ? "registered" : "not registered");
	printf("foo is %s.\n", isTypedefName("foo") ? "registered" : "not registered");
	
	deleteTypedefNameAt(1);
	printf("Deleted 'typedef-name's at scope 1.\n");
	
	printf("hoge is %s.\n", isTypedefName("hoge") ? "registered" : "not registered");
	printf("bar is %s.\n", isTypedefName("bar") ? "registered" : "not registered");
	printf("foo is %s.\n", isTypedefName("foo") ? "registered" : "not registered");
	
	printf("=======\n");
	
	if (registerTypedefName("hoge", 1) == -1)
	{
		printf("Cannot registered 'hoge' at scope 1.");
		return -1;
	}
	printf("Registered 'hoge' at scope 1.\n");
	
	if (registerTypedefName("hoge", 2) == -1)
	{
		printf("Cannot registered 'hoge' at scope 2.");
		return -1;
	}
	printf("Registered 'hoge' at scope 2.\n");
	
	if (registerTypedefName("hoge", 3) == -1)
	{
		printf("Cannot registered 'hoge' at scope 3.");
		return -1;
	}
	printf("Registered 'hoge' at scope 3.\n");
	
	if (registerTypedefName("foo", 1) == -1)
	{
		printf("Cannot registered 'foo' at scope 1.");
		return -1;
	}
	printf("Registered 'foo' at scope 1.\n");
	
	printf("hoge is %s.\n", isTypedefName("hoge") ? "registered" : "not registered");
	printf("bar is %s.\n", isTypedefName("bar") ? "registered" : "not registered");
	printf("foo is %s.\n", isTypedefName("foo") ? "registered" : "not registered");
	
	freeTypedefDictionary();
	
	return 0;
}
*/
